package org.fuseri.modulecertificate.service;

import jakarta.validation.Valid;
import org.fuseri.model.dto.certificate.CertificateCreateDto;
import org.fuseri.model.dto.certificate.CertificateDto;
import org.fuseri.model.dto.certificate.CertificateSimpleDto;
import org.fuseri.model.dto.common.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * This class represents a RESTful controller for Certificate module.
 * It handles incoming HTTP requests for generating and retrieving course certificates,
 * and delegates them to the appropriate service method.
 */
@RestController
@RequestMapping("/certificates")
public class CertificateController {

    private final CertificateFacade certificateFacade;

    @Autowired
    public CertificateController(CertificateFacade certificateFacade) {
        this.certificateFacade = certificateFacade;
    }


    /**
     * Generates certificate for specified user and course.
     *
     * @param certificateCreateDto Dto with data used for generating certificate
     * @return CertificateDto with data of generated certificate
     */
    @PostMapping("/generate")
    public CertificateSimpleDto generate(@Valid @RequestBody CertificateCreateDto certificateCreateDto) {
        return certificateFacade.generate(certificateCreateDto);
    }

    /**
     * Retrieves previously generated certificate.
     *
     * @param id ID of certificate to be retrieved
     * @return CertificateDto with data of previously generated certificate with specified ID
     */
    @GetMapping("/find")
    public CertificateSimpleDto find(@RequestParam Long id) {
        return certificateFacade.findById(id);
    }

    /**
     * Retrieves previously generated certificates for user.
     *
     * @param userId ID of user to retrieve certificates for.
     * @return List of CertificateDto objects with previously generated certificates
     * for specified User.
     */
    @GetMapping("/findForUser")
    public List<CertificateSimpleDto> findForUser(@RequestParam Long userId) {
        return certificateFacade.findByUserId(userId);
    }

    /**
     * Retrieves previously generated certificate ID.
     *
     * @param userId   ID of user to retrieve certificates for.
     * @param courseId ID of course to retrieve certificates for.
     * @return List of CertificateDto objects with previously generated certificates
     * for specified User and Course.
     */
    @GetMapping("/findForUserAndCourse")
    public List<CertificateSimpleDto> findForUserAndCourse(@RequestParam Long userId, @RequestParam Long courseId) {
        return certificateFacade.findByUserIdAndCourseId(userId, courseId);
    }

    /**
     * Retrieves previously generated certificate ID.
     *
     * @param id Id of certificate to be deleted.
     */
    @DeleteMapping("/delete")
    public void delete(@RequestParam Long id) {
        certificateFacade.deleteCertificate(id);
    }

    /**
     * Find certificates and return them in a paginated format
     *
     * @return a Result object containing a list of CertificateDto objects and pagination information
     */
    @GetMapping("/findAll")
    public Page<CertificateSimpleDto> findAllCertificates(@RequestBody Pageable pageable) {
        return certificateFacade.findAll(pageable);
    }
}
