package org.fuseri.moduleexercise.exercise;

import jakarta.persistence.EntityNotFoundException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.PositiveOrZero;
import org.fuseri.model.dto.common.Result;
import org.fuseri.model.dto.exercise.ExerciseCreateDto;
import org.fuseri.model.dto.exercise.ExerciseDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

/**
 * Represent a REST API controller for exercises
 * Handle HTTP requests related to exercises
 */
@RestController
@RequestMapping("/exercises")
public class ExerciseController {

    private final ExerciseService service;

    private final ExerciseMapper mapper;

    /**
     * Constructor for AnswerController
     *
     * @param service the service responsible for handling exercise-related logic
     * @param mapper  the mapper responsible for converting between DTOs and entities
     */
    @Autowired
    public ExerciseController(ExerciseService service, ExerciseMapper mapper) {
        this.service = service;
        this.mapper = mapper;
    }

    /**
     * Create a new answer for the given question ID
     *
     * @param dto the ExerciseCreateDto object containing information about the exercise to create
     * @return an ExerciseDto object representing the newly created exercise
     */
    @PostMapping
    public ExerciseDto create(@Valid @RequestBody ExerciseCreateDto dto) {
        Exercise exercise = mapper.fromCreateDto(dto);
        return mapper.toDto(service.create(exercise));
    }

    /**
     * Find an exercise by ID
     *
     * @param id the ID of the exercise to find
     * @return an ExerciseDto object representing the found exercise
     */
    @GetMapping("/{id}")
    public ExerciseDto find(@NotBlank @PathVariable String id) {
        return mapper.toDto(service.find(id));
    }

    /**
     * Find exercises and return them in a paginated format
     *
     * @param page the page number of the exercises to retrieve
     * @return a Result object containing a list of ExerciseDto objects and pagination information
     */
    @GetMapping
    public Result<ExerciseDto> findAll(@PositiveOrZero @RequestParam int page) {
        Page<Exercise> exercise = service.findAll(page);
        return mapper.toResult(exercise);
    }

    /**
     * Find exercises that mach filters and return them in paginated format
     *
     * @param page       the page number of the exercises to retrieve
     * @param courseId   the id of the course to filter by
     * @param difficulty the difficulty level to filter by
     * @return a Result object containing a list of filtered ExerciseDto objects woth pagination information
     */
    @GetMapping("filter")
    public Result<ExerciseDto> findPerDifficultyPerCourse(
            @PositiveOrZero @RequestParam int page, @NotBlank @RequestParam String courseId,
            @PositiveOrZero @RequestParam int difficulty) {
        Page<Exercise> exercise = service.findPerDifficultyPerCourse(page, courseId, difficulty);
        return mapper.toResult(exercise);
    }

    /**
     * Update an exercise with id
     *
     * @param id  the ID of the exercise to update
     * @param dto the ExerciseCreateDto object containing information about the exercise to update
     * @return an ExerciseDto object representing the updated exercise
     * @throws ResponseStatusException invalid exercise id
     */

    @PutMapping("/{id}")
    public ExerciseDto update(@NotBlank @PathVariable String id, @Valid @RequestBody ExerciseCreateDto dto) {
        Exercise exercise = mapper.fromCreateDto(dto);
        exercise.setId(id);

        try {
            return mapper.toDto(service.update(exercise));
        } catch (IllegalArgumentException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        } catch (EntityNotFoundException e) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, e.getMessage());
        }
    }

    /**
     * Delete an exercise with exerciseId
     *
     * @param id the ID of the exercise to delete
     */
    @DeleteMapping("/{id}")
    public void delete(@NotBlank @PathVariable String id) {
        service.delete(id);
    }

}
