package org.fuseri.modulelanguageschool.course;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.fuseri.model.dto.course.CourseCreateDto;
import org.fuseri.model.dto.course.CourseDto;
import org.fuseri.model.dto.course.LanguageTypeDto;
import org.fuseri.model.dto.course.ProficiencyLevelDto;
import org.fuseri.model.dto.user.AddressDto;
import org.fuseri.model.dto.user.UserDto;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.PageImpl;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
public class CourseControllerTest {

    private final CourseCreateDto courseCreateDto = new CourseCreateDto("english b2 course", 10, LanguageTypeDto.ENGLISH, ProficiencyLevelDto.B2);
    private final CourseDto courseDto = new CourseDto("english b2 course", 10, LanguageTypeDto.ENGLISH, ProficiencyLevelDto.B2);
    @Autowired
    ObjectMapper objectMapper;
    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private CourseController courseController;

    public static String asJsonString(final Object obj) {
        try {
            return new ObjectMapper().writeValueAsString(obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Test
    void createCourse() throws Exception {
        Mockito.when(courseController.create(ArgumentMatchers.isA(CourseCreateDto.class))).thenReturn(courseDto);
        mockMvc.perform(post("/courses")
                        .content(asJsonString(courseCreateDto))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.name").value("english b2 course"))
                .andExpect(jsonPath("$.capacity").value(10))
                .andExpect(jsonPath("$.language").value("ENGLISH"))
                .andExpect(jsonPath("$.proficiency").value("B2"))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void createInvalidCourse() throws Exception {
        CourseCreateDto invalidCourseCreateDto =
                new CourseCreateDto(null, null, null, null);
        Mockito.when(courseController.create(ArgumentMatchers.isA(CourseCreateDto.class))).thenReturn(courseDto);
        mockMvc.perform(post("/courses")
                        .content(asJsonString(invalidCourseCreateDto))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void createCourseWithoutParameter() throws Exception {
        Mockito.when(courseController.create(ArgumentMatchers.isA(CourseCreateDto.class))).thenReturn(courseDto);
        mockMvc.perform(post("/courses"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCourse() throws Exception {
        Long id = 0L;
        Mockito.when(courseController.find(id)).thenReturn(courseDto);
        mockMvc.perform(get("/courses/find/" + id))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.name").value("english b2 course"))
                .andExpect(jsonPath("$.capacity").value(10))
                .andExpect(jsonPath("$.language").value("ENGLISH"))
                .andExpect(jsonPath("$.proficiency").value("B2"))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void findCourseWithoutId() throws Exception {
        Mockito.when(courseController.find(ArgumentMatchers.anyLong())).thenReturn(courseDto);
        mockMvc.perform(get("/courses/find/"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findAll() throws Exception {
        int page = 0;
        Mockito.when(courseController.findAll(page)).thenReturn(new PageImpl<>(new ArrayList<>()));
        String response = mockMvc.perform(get("/courses/findAll").param("page", Integer.toString(page)))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();

        assertTrue(response.contains("\"content\":[]"));
    }

    @Test
    void findAllWithoutPage() throws Exception {
//        Mockito.when(courseController.findAll(ArgumentMatchers.anyInt())).thenReturn(new PageImpl<>(new ArrayList<>()));
        mockMvc.perform(get("/courses/findAll"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findAllByLang() throws Exception {
        int page = 0;
        LanguageTypeDto lang = LanguageTypeDto.ENGLISH;
        Mockito.when(courseController.findAll(lang)).thenReturn(new ArrayList<>());
        String response = mockMvc.perform(get("/courses/findAllByLang")
                        .param("page", Integer.toString(page))
                        .param("lang", lang.toString()))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();

        assertThat("response", response, is("[]"));
    }

    @Test
    void findAllByLangWithoutLang() throws Exception {
        mockMvc.perform(get("/courses/findAllByLang"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findAllByLangProf() throws Exception {
        LanguageTypeDto lang = LanguageTypeDto.ENGLISH;
        ProficiencyLevelDto proficiencyLevel = ProficiencyLevelDto.A1;
        Mockito.when(courseController.findAll(lang, proficiencyLevel)).thenReturn(new ArrayList<>());
        String response = mockMvc.perform(get("/courses/findAllByLangProf")
                        .param("lang", lang.toString())
                        .param("prof", proficiencyLevel.toString()))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();

        assertThat("response", response, is("[]"));
    }

    @Test
    void findAllByLangProfWithoutParameters() throws Exception {
        Mockito.when(courseController.findAll(
                        ArgumentMatchers.isA(LanguageTypeDto.class),
                        ArgumentMatchers.isA(ProficiencyLevelDto.class)))
                .thenReturn(new ArrayList<>());
        mockMvc.perform(get("/courses/findAllByLangProf"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findAllByLangProfWithoutLangProf() throws Exception {
        Mockito.when(courseController.findAll(
                        ArgumentMatchers.isA(LanguageTypeDto.class),
                        ArgumentMatchers.isA(ProficiencyLevelDto.class)))
                .thenReturn(new ArrayList<>());
        String page = "0";
        mockMvc.perform(get("/courses/findAllByLangProf").param("page", page))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void updateCourse() throws Exception {
        Long id = 0L;
        Mockito.when(courseController.update(ArgumentMatchers.eq(id),
                        ArgumentMatchers.isA(CourseCreateDto.class)))
                .thenReturn(courseDto);

        mockMvc.perform(put("/courses/update/" + id)
                        .content(asJsonString(courseDto))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.name").value("english b2 course"))
                .andExpect(jsonPath("$.capacity").value(10))
                .andExpect(jsonPath("$.language").value("ENGLISH"))
                .andExpect(jsonPath("$.proficiency").value("B2"))
                .andExpect(jsonPath("$.id").value(courseDto.getId()))
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void updateCourseWithoutParameter() throws Exception {
        Mockito.when(courseController.update(ArgumentMatchers.anyLong(),
                        ArgumentMatchers.isA(CourseCreateDto.class)))
                .thenReturn(courseDto);
        mockMvc.perform(put("/courses/update"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void deleteCourse() throws Exception {
        Long id = 0L;
        Mockito.doNothing().when(courseController).delete(id);

        mockMvc.perform(delete("/courses/delete/" + id))
                .andExpect(status().isOk());
    }

    @Test
    void deleteCourseWithoutParameter() throws Exception {
        Mockito.doNothing().when(courseController).delete(ArgumentMatchers.anyLong());

        mockMvc.perform(delete("/courses/delete/"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void enrolCourse() throws Exception {
        Long id = 0L;
        UserDto student = new UserDto("novakovat", "novakova@gamil.com", "Tereza",
                "Nováková", new AddressDto());

        CourseDto courseDtoWithStudent = new CourseDto("english b2 course", 10, LanguageTypeDto.ENGLISH, ProficiencyLevelDto.B2);
        courseDtoWithStudent.setStudentIds(new ArrayList<>(List.of(student.getId())));

        Mockito.when(courseController.enrol(ArgumentMatchers.eq(id),
                ArgumentMatchers.isA(UserDto.class))).thenReturn(courseDtoWithStudent);

        mockMvc.perform(patch("/courses/enrol/" + id)
                        .content(asJsonString(student))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.name").value("english b2 course"))
                .andExpect(jsonPath("$.capacity").value(10))
                .andExpect(jsonPath("$.language").value("ENGLISH"))
                .andExpect(jsonPath("$.proficiency").value("B2"))
                .andExpect(jsonPath("$.studentIds").exists())
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void enrolCourseWithoutUserParameter() throws Exception {
        String id = "0";
        Mockito.when(courseController.enrol(ArgumentMatchers.anyLong(),
                        ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(courseDto);
        mockMvc.perform(patch("/courses/enrol/" + id))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void enrolCourseWithoutCourseIdParameter() throws Exception {
        Mockito.when(courseController.enrol(ArgumentMatchers.anyLong(),
                        ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(courseDto);
        UserDto student = new UserDto("novakovat", "novakova@gamil.com", "Tereza",
                "Nováková", new AddressDto());

        mockMvc.perform(patch("/courses/enrol/")
                        .content(asJsonString(student))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void expelCourse() throws Exception {
        Long id = 0L;
        Mockito.when(courseController.expel(ArgumentMatchers.eq(id),
                        ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(courseDto);

        UserDto student = new UserDto("novakovat", "novakova@gamil.com", "Tereza",
                "Nováková", new AddressDto());

        mockMvc.perform(patch("/courses/expel/" + id)
                        .content(asJsonString(student))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.name").value("english b2 course"))
                .andExpect(jsonPath("$.capacity").value(10))
                .andExpect(jsonPath("$.language").value("ENGLISH"))
                .andExpect(jsonPath("$.proficiency").value("B2"))
                .andExpect(jsonPath("$.studentIds").isEmpty())
                .andReturn().getResponse().getContentAsString();
    }

    @Test
    void expelCourseWithoutUserParameter() throws Exception {
        Long id = 0L;
        Mockito.when(courseController.expel(ArgumentMatchers.eq(id),
                        ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(courseDto);

        mockMvc.perform(patch("/courses/expel/" + id))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void deleteCourseWithoutCourseIdParameter() throws Exception {
        Mockito.when(courseController.expel(ArgumentMatchers.anyLong(),
                        ArgumentMatchers.isA(UserDto.class)))
                .thenReturn(courseDto);
        UserDto student = new UserDto("novakovat", "novakova@gamil.com", "Tereza",
                "Nováková", new AddressDto());

        mockMvc.perform(patch("/courses/expel/")
                        .content(asJsonString(student))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }
}
